﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace imlac
{
    class EntryPoint
    {
        static void Main(string[] args)
        {
            ImlacSystem system = new ImlacSystem();
            Debugger debugger = new Debugger(system);

            _state = SystemExecutionState.Debugging;

            Console.CancelKeyPress += new ConsoleCancelEventHandler(OnBreak);

            PrintHerald();

            while (_state != SystemExecutionState.Quit)
            {
                try
                {
                    switch (_state)
                    {
                        case SystemExecutionState.Halted:
                        case SystemExecutionState.Debugging:
                            _state = debugger.Prompt();
                            break;

                        case SystemExecutionState.SingleStep:
                            system.SingleStep();
                            system.Display.RenderCurrent();
                            _state = SystemExecutionState.Debugging;
                            break;

                        case SystemExecutionState.SingleFrame:
                            system.SingleStep();

                            if (system.DisplayProcessor.FrameLatch)
                            {
                                _state = SystemExecutionState.Debugging;
                            }
                            break;

                        case SystemExecutionState.UntilDisplayStart:
                            system.SingleStep();

                            if (system.DisplayProcessor.State == ProcessorState.Running)
                            {
                                _state = SystemExecutionState.Debugging;
                            }
                            break;

                        case SystemExecutionState.Running:
                            system.SingleStep();

                            if (system.Processor.State == ProcessorState.Halted)
                            {
                                Console.WriteLine("Main processor halted at {0}", Helpers.ToOctal(system.Processor.PC));
                                _state = SystemExecutionState.Debugging;
                            }
                            break;
                    }
                }
                catch(Exception e)
                {
                    Console.WriteLine("Internal error during execution: {0}", e.Message);
                    _state = SystemExecutionState.Debugging;
                }
            }
        }

        static void OnBreak(object sender, ConsoleCancelEventArgs e)
        {
            _state = SystemExecutionState.Debugging;
            e.Cancel = true;
        }

        static void PrintHerald()
        {
            Console.WriteLine("sImlac v0.0.  (c) 2013 j. dersch.");
            Console.WriteLine();
        }

        private static SystemExecutionState _state;
    }
}
