﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using imlac.IO;

namespace imlac
{
    public enum DrawingMode
    {
        Off,
        Normal,
        Dotted,
        Hidden,
        SGR1,      // reduced intensity
        Point,     // increased intensity
    }    

    /// <summary>
    /// IImlacConsole provides the interface for the components making up a standard
    /// "console" setup, which at the moment includes only the Display and Keyboard,
    /// but could be extended.
    /// </summary>
    public interface IImlacConsole
    {
        /// <summary>
        /// Indicates whether a key is currently pressed.
        /// </summary>
        bool IsKeyPressed
        {
            get;
        }

        /// <summary>
        /// Indicates the currently pressed key (if IsKeyPressed is true)
        /// </summary>
        ImlacKey Key
        {
            get;
        }

        /// <summary>
        /// Indicates any modifier bits (ctrl, shift) being pressed.
        /// </summary>
        ImlacKeyModifiers KeyModifiers
        {
            get;
        }

        /// <summary>
        /// Indicates the status of the front-panel data switches
        /// </summary>
        ushort DataSwitches
        {
            get;
        }

        void UnlatchKey();

        void ClearDisplay();

        void MoveAbsolute(uint x, uint y, DrawingMode mode);

        void DrawPoint(uint x, uint y);

        void RenderCurrent();

        void FrameDone();
    }
}
