﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace imlac.IO
{
    public class TTY : IIOTDevice
    {
        public TTY(ImlacSystem system)
        {
            _system = system;
            Reset();
        }

        public void Reset()
        {
            _dataSendReady = true;
            _imageIndex = 0;
            _dataReady = false;
            _clocks = 0;
        }
        
        public void LoadImage(string path)
        {            
            _imageIndex = 0;
            _dataReady = false;                    

            FileStream fs = File.OpenRead(path);
            _imageData = new byte[fs.Length];

            fs.Read(_imageData, 0, (int)fs.Length);

            fs.Close();
        }

        public void Clock()
        {
            if (_imageData != null)
            {
                _clocks++;

                if (_clocks > _dataClocks)
                {
                    _clocks = 0;

                    if (_imageIndex < _imageData.Length)
                    {
                        _dataReady = true;
                        _data = _imageData[_imageIndex];
                        _imageIndex++;
                        Console.Write("."); 
                    }
                    else
                    {
                        _dataReady = false;
                    }
                }
            }
        }

        public bool DataReady
        {
            get { return _dataReady; }
        }

        public bool DataSendReady
        {
            get { return _dataSendReady; }
        }

        public int[] GetHandledIOTs()
        {
            return _handledIOTs;
        }

        public void ExecuteIOT(int iotCode)
        {
            switch (iotCode)
            {
                case 0x19:  // RRB - TTY read     
                    Trace.Log(LogType.TTY, "TTY read {0}", Helpers.ToOctal(_data));
                    _system.Processor.AC |= _data;                    
                    break;

                case 0x1a:  // RCF - Clear TTY status
                    _dataReady = false;
                    break;

                case 0x1b:  // RRC - Read and clear status
                    Trace.Log(LogType.TTY, "TTY read {0}, status cleared.", Helpers.ToOctal(_data));                    
                    _dataReady = false;
                    _system.Processor.AC |= _data;
                    break;

                default:
                    Trace.Log(LogType.TTY, "Stub: TTY xmit op", Helpers.ToOctal(_data));                    
                    break;
            }
        }

        private readonly int[] _handledIOTs = { 0x9, 0x19, 0x1a, 0x1b, 0x21, 0x22 };

        private int _imageIndex;
        private byte[] _imageData;
        private bool _dataReady;
        private bool _dataSendReady;
        private byte _data;

        private int _clocks;
        private readonly int _dataClocks = 1000;

        private ImlacSystem _system;
    }
}
