﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using imlac.IO;

namespace imlac
{
    public enum SystemExecutionState
    {
        Debugging,
        Halted,
        SingleStep,
        SingleFrame,
        UntilDisplayStart,
        Running,
        Quit
    }

    public class ImlacSystem
    {
        public ImlacSystem()
        {            
            _display = new SDLConsole(1);
            _memory = new Memory(this);
            _paperTapeReader = new PaperTapeReader(this);
            _tty = new TTY(this);
            _keyboard = new Keyboard(this);
            _clock = new AddressableClock(this);
            _interruptFacility = new InterruptFacility(this);
            _displayProcessor = new DisplayProcessor(this);
            _processor = new Processor(this);

            // Register IOT devices
            _processor.RegisterDeviceIOTs(_displayProcessor);
            _processor.RegisterDeviceIOTs(_paperTapeReader);
            _processor.RegisterDeviceIOTs(_tty);
            _processor.RegisterDeviceIOTs(_keyboard);
            _processor.RegisterDeviceIOTs(_clock);
            _processor.RegisterDeviceIOTs(_interruptFacility);
        }

        public void Reset()
        {
            _paperTapeReader.Reset();
            _tty.Reset();
            _keyboard.Reset();
            _clock.Reset();
            _interruptFacility.Reset();
            _displayProcessor.Reset();
            _processor.Reset();
        }

        public Memory Memory
        {
            get { return _memory; }
        }

        public Processor Processor
        {
            get { return _processor; }
        }

        public DisplayProcessor DisplayProcessor
        {
            get { return _displayProcessor; }
        }

        public IImlacConsole Display
        {
            get { return _display; }
        }

        public PaperTapeReader PaperTapeReader
        {
            get { return _paperTapeReader; }
        }

        public TTY TTY
        {
            get { return _tty; }
        }

        public Keyboard Keyboard
        {
            get { return _keyboard; }
        }

        public AddressableClock Clock
        {
            get { return _clock; }
        }

        public InterruptFacility InterruptFacility
        {
            get { return _interruptFacility; }
        }

        public void SingleStep()
        {
            _processor.Clock();
            _displayProcessor.Clock();
            _paperTapeReader.Clock();
            _tty.Clock();
            _keyboard.Clock();
            _clock.Clock();

            // interrupts last so that devices that raise interrupts get clocked first
            _interruptFacility.Clock();
        }

        private Processor _processor;
        private DisplayProcessor _displayProcessor;
        private IImlacConsole _display;
        private Memory _memory;
        private PaperTapeReader _paperTapeReader;
        private TTY _tty;
        private Keyboard _keyboard;
        private InterruptFacility _interruptFacility;
        private AddressableClock _clock;
    }
}
